/*
 * $Log: logsrv.c,v $
 * Revision 1.17  2021-04-05 21:58:03+05:30  Cprogrammer
 * fixed compilation errors
 *
 * Revision 1.16  2021-03-15 12:50:29+05:30  Cprogrammer
 * include filewrt.h
 *
 * Revision 1.15  2021-03-15 11:32:19+05:30  Cprogrammer
 * added sockread() function
 *
 * Revision 1.14  2020-06-21 12:49:10+05:30  Cprogrammer
 * quench rpmlint
 *
 * Revision 1.13  2018-08-21 19:38:58+05:30  Cprogrammer
 * fix for rpc.h on openSUSE tumbleweed
 *
 * Revision 1.12  2018-08-20 13:13:04+05:30  Cprogrammer
 * added tirpc/rpc inclusion
 *
 * Revision 1.11  2017-03-07 19:35:08+05:30  Cprogrammer
 * added CRYPTO_POLICY_NON_COMPLIANCE
 *
 * Revision 1.10  2016-06-21 13:30:57+05:30  Cprogrammer
 * use SSL_set_cipher_list as part of crypto-policy-compliance
 *
 * Revision 1.9  2014-04-17 11:27:43+05:30  Cprogrammer
 * added sys/param.h
 *
 * Revision 1.8  2013-05-15 00:39:03+05:30  Cprogrammer
 * added SSL encryption
 *
 * Revision 1.7  2013-03-03 23:36:52+05:30  Cprogrammer
 * read MAXHOSTNAMELEN bytes for host
 *
 * Revision 1.6  2013-02-21 22:46:05+05:30  Cprogrammer
 * use 0 as IP address for localhost
 *
 * Revision 1.5  2013-02-11 23:03:10+05:30  Cprogrammer
 * added bytes read to statusfile
 *
 * Revision 1.4  2012-09-19 11:07:08+05:30  Cprogrammer
 * removed unused variable
 *
 * Revision 1.3  2012-09-18 17:08:47+05:30  Cprogrammer
 * removed syslog
 *
 * Revision 1.2  2012-09-18 14:55:24+05:30  Cprogrammer
 * removed stdio.h
 *
 * Revision 1.1  2012-09-18 13:23:48+05:30  Cprogrammer
 * Initial revision
 *
 */
#include <getopt.h>
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif
#ifdef HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif
#ifdef HAVE_RPC_RPC_H
#include <rpc/rpc.h>
#endif
#ifdef HAVE_RPC_TYPES_H
#include <rpc/types.h>
#endif
/*- tirpc */
#ifdef HAVE_TIRPC
#ifndef HAVE_RPC_RPC_H
#include <tirpc/rpc/rpc.h>
#endif
#ifndef HAVE_RPC_TYPES_H
#include <tirpc/rpc/types.h>
#endif
#endif

#include <signal.h>
#include <errno.h>
#ifdef HAVE_SSL
#include <openssl/ssl.h>
#include <openssl/err.h>
#endif
#ifdef HAVE_INDIMAIL
#include <indimail/filewrt.h>
#endif
#ifdef HAVE_QMAIL
#include <qmail/getEnvConfig.h>
#endif
#include "tls.h"
#include "common.h"

/*-
program RPCLOG
{
	version CLOGVERS
	{
		int	SEND_MESSAGE( string ) = 1;
	} = 1;
} = 0x20000001;
-*/


#define RPCLOG ((u_long)0x20000001)
#define CLOGVERS ((u_long)1)
#define SEND_MESSAGE ((u_long)1)

#define MAXBUF    8192
#define PIDFILE   "/tmp/logsrv.pid"
#define PORT      "logsrv"
#define STATUSDIR PREFIX"/tmp/"

#ifndef	lint
static char     sccsid[] = "$Id: logsrv.c,v 1.17 2021-04-05 21:58:03+05:30 Cprogrammer Exp mbhangui $";
#endif

#ifdef __STDC__
int             main(int, char **);
int             server(int);
int             log_msg(char **);
int            *send_message_1(char **, CLIENT *);
#else
int             main();
int             server();
int             log_msg();
int            *send_message_1();
#endif
void            SigTerm();
static void     SigChild();
#ifdef HAVE_SSL
static void     SigHup();
int             translate(SSL *, int, int, int, unsigned int);
#endif

char           *progname, *statusdir, *loghost;
int             sighupflag, logsrvpid, rpcflag;
char            statusfile[MAXBUF];
CLIENT         *cl;
#ifdef HAVE_SSL
static int      usessl = 0;
static char    *certfile;
SSL_CTX        *ctx = (SSL_CTX *) 0;
#endif

#ifdef HAVE_SSL
char            tbuf[2048];

static int
ssl_write(SSL *ssl, char *buf, int len)
{
	int             w;

	while (len)
	{
		if ((w = SSL_write(ssl, buf, len)) == -1)
		{
			if (errno == EINTR)
				continue;
			return -1;	/*- note that some data may have been written */
		}
		if (w == 0);	/*- luser's fault */
		buf += w;
		len -= w;
	}
	return 0;
}

int
sockwrite(fd, wbuf, len)
	int             fd;
	char           *wbuf;
	int             len;
{
	char           *ptr;
	int             rembytes, wbytes;

	for (ptr = wbuf, rembytes = len; rembytes;)
	{
		for (;;)
		{
			errno = 0;
			if ((wbytes = write(fd, ptr, rembytes)) == -1)
			{
#ifdef ERESTART
				if (errno == EINTR || errno == ERESTART)
#else
				if (errno == EINTR)
#endif
					continue;
				return (-1);
			} else
			if (!wbytes)
				return(0);
			break;
		}
		ptr += wbytes;
		rembytes -= wbytes;
	}
	return (len);
}

int
translate(SSL *ssl, int out, int clearout, int clearerr, unsigned int iotimeout)
{
	fd_set          rfds;	/*- File descriptor mask for select -*/
	struct timeval  timeout;
	int             flagexitasap;
	int             sslin;
	int             retval, n, r;

	timeout.tv_sec = iotimeout;
	timeout.tv_usec = 0;
	flagexitasap = 0;
	if ((sslin = SSL_get_rfd(ssl)) == -1)
	{
		fprintf(stderr, "translate: unable to set up SSL connection\n");
		while ((n = ERR_get_error()))
			fprintf(stderr, "translate: %s\n", ERR_error_string(n, 0));
		return (-1);
	}
	if (SSL_accept(ssl) <= 0)
	{
		fprintf(stderr, "translate: unable to accept SSL connection\n");
		while ((n = ERR_get_error()))
			fprintf(stderr, "translate: %s\n", ERR_error_string(n, 0));
		return (-1);
	}
	while (!flagexitasap)
	{
		FD_ZERO(&rfds);
		FD_SET(sslin, &rfds);
		FD_SET(clearout, &rfds);
		FD_SET(clearerr, &rfds);
		if ((retval = select(clearerr > sslin ? clearerr + 1 : sslin + 1, &rfds, (fd_set *) NULL, (fd_set *) NULL, &timeout)) < 0)
		{
#ifdef ERESTART
			if (errno == EINTR || errno == ERESTART)
#else
			if (errno == EINTR)
#endif
				continue;
			fprintf(stderr, "translate: %s\n", strerror(errno));
			return (-1);
		} else
		if (!retval)
		{
			fprintf(stderr, "translate: timeout reached without input [%ld sec]\n", timeout.tv_sec);
			return (-1);
		}
		if (FD_ISSET(sslin, &rfds))
		{
			/*- data on sslin */
			if ((n = SSL_read(ssl, tbuf, sizeof(tbuf))) < 0)
			{
				fprintf(stderr, "translate: unable to read from network: %s\n", strerror(errno));
				flagexitasap = 1;
			} else
			if (n == 0)
				flagexitasap = 1;
			else
			if ((r = sockwrite(out, tbuf, n)) < 0)
			{
				fprintf(stderr, "translate: unable to write to client: %s\n", strerror(errno));
				return (-1);
			}
		}
		if (FD_ISSET(clearout, &rfds))
		{
			/*- data on clearout */
			if ((n = read(clearout, tbuf, sizeof(tbuf))) < 0)
			{
				fprintf(stderr, "translate: unable to read from client: %s\n", strerror(errno));
				return (-1);
			} else
			if (n == 0)
				flagexitasap = 1;
			if ((r = ssl_write(ssl, tbuf, n)) < 0)
			{
				fprintf(stderr, "translate: unable to write to network: %s\n", strerror(errno));
				return (-1);
			}
		}
		if (FD_ISSET(clearerr, &rfds))
		{
			/*- data on clearerr */
			if ((n = read(clearerr, tbuf, sizeof(tbuf))) < 0)
			{
				fprintf(stderr, "translate: unable to read from client: %s\n", strerror(errno));
				return (-1);
			} else
			if (n == 0)
				flagexitasap = 1;
			if ((r = ssl_write(ssl, tbuf, n)) < 0)
			{
				fprintf(stderr, "translate: unable to write to network: %s\n", strerror(errno));
				return (-1);
			}
		}
	} /*- while (!flagexitasap) */
	return (0);
}

SSL_CTX *
load_certificate(char *certfile)
{
	SSL_CTX        *myctx = (SSL_CTX *) 0;
#ifdef CRYPTO_POLICY_NON_COMPLIANCE
	char           *cipher;
#endif

    /* setup SSL context (load key and cert into ctx) */
	if (!(myctx = SSL_CTX_new(SSLv23_server_method())))
	{
		fprintf(stderr, "SSL_CTX_new: unable to create SSL context: %s\n",
			ERR_error_string(ERR_get_error(), 0));
		return ((SSL_CTX *) 0);
	}
	/* set prefered ciphers */
#ifdef CRYPTO_POLICY_NON_COMPLIANCE
	cipher = getenv("SSL_CIPHER");
	if (cipher && !SSL_CTX_set_cipher_list(myctx, cipher))
	{
		fprintf(stderr, "SSL_CTX_set_cipher_list: unable to set cipher list: %s\n",
			ERR_error_string(ERR_get_error(), 0));
		SSL_CTX_free(myctx);
		return ((SSL_CTX *) 0);
	}
#endif
	if (SSL_CTX_use_certificate_chain_file(myctx, certfile))
	{
		if (SSL_CTX_use_RSAPrivateKey_file(myctx, certfile, SSL_FILETYPE_PEM) != 1)
		{
			fprintf(stderr, "SSL_CTX_use_RSAPrivateKey: unable to load RSA private key: %s\n",
				ERR_error_string(ERR_get_error(), 0));
			SSL_CTX_free(myctx);
			return ((SSL_CTX *) 0);
		}
		if (SSL_CTX_use_certificate_file(myctx, certfile, SSL_FILETYPE_PEM) != 1)
		{
			fprintf(stderr, "SSL_CTX_use_certificate_file: unable to load certificate: %s\n",
			ERR_error_string(ERR_get_error(), 0));
			SSL_CTX_free(myctx);
			return ((SSL_CTX *) 0);
		}
	}
	return (myctx);
}

static void
SigHup(void)
{
	fprintf(stderr, "%d: logsrv received SIGHUP\n", getpid());
	ctx = load_certificate(certfile);
	signal(SIGHUP, (void(*)()) SigHup);
	errno = EINTR;
	return;
}
#endif

int
get_options(int argc, char **argv, int *foreground, int *silent)
{
	int             c, errflag = 0;

#ifdef HAVE_SSL
	certfile = 0;
#endif
#ifdef HAVE_SSL
	while (!errflag && (c = getopt(argc, argv, "fsr:n:")) != -1) {
#else
	while (!errflag && (c = getopt(argc, argv, "fsr:")) != -1) {
#endif
		switch (c)
		{
		case 's':
			*silent = 1;
			break;
		case 'f':
			*foreground = 1;
			break;
		case 'r':
			rpcflag = 1;
			loghost = optarg;
			break;
#ifdef HAVE_SSL
		case 'n':
			usessl = 1;
			certfile = optarg;
			break;
#endif
		default:
			errflag = 1;
			break;
		}
	}
	if (errflag)
		return (1);
	return (0);
}

static void
SigChild(void)
{
	int             status;
	pid_t           pid;

	for (;(pid = waitpid(-1, &status, WNOHANG | WUNTRACED));)
	{
#ifdef ERESTART
		if (pid == -1 && (errno == EINTR || errno == ERESTART))
#else
		if (pid == -1 && errno == EINTR)
#endif
			continue;
		break;
	} /*- for (; pid = waitpid(-1, &status, WNOHANG | WUNTRACED);) -*/
	(void) signal(SIGCHLD, (void (*)()) SigChild);
	return;
}

void
usage(char *pname)
{
#ifdef HAVE_SSL
	filewrt(2, "Usage: %s [-s] [-f] [-n certfile] [-r rpchost]\n", pname);
#else
	filewrt(2, "Usage: %s [-s] [-f] [-r rpchost]\n", pname);
#endif
	return;
}

int             log_timeout;

int
main(int argc, char **argv)
{
	FILE           *pidfp;
	char           *ptr;
	int             pid, bindfd, sfd, foreground = 0, silent = 0, len;
	struct sockaddr_in remotaddr;	/* for peer socket address */
	int             inaddrlen = sizeof(struct sockaddr_in);
	struct linger   linger;
#ifdef HAVE_SSL
	BIO            *sbio;
	SSL            *ssl;
	int             r, status, retval, pi1[2], pi2[2], pi3[2];
#endif

	if ((progname = strrchr(argv[0], '/')))
		progname++;
	else
		progname = argv[0];
	if (get_options(argc, argv, &foreground, &silent)) {
		usage(progname);
		return (1);
	}
#ifdef HAVE_SSL
	if (usessl == 1)
	{
		if (access(certfile, F_OK))
		{
			fprintf(stderr, "missing certficate: %s: %s\n", certfile, strerror(errno));
			return (1);
		}
		signal(SIGHUP, (void (*)()) SigHup);
		SSL_library_init();
		if (!(ctx = load_certificate(certfile)))
			return (1);
	}
#endif
	getEnvConfigInt(&log_timeout, "LOGSRV_TIMEOUT", 120);
	if (!(statusdir = getenv("STATUSDIR")))
		statusdir = STATUSDIR;
	if (!foreground) {
		if ((pidfp = fopen(PIDFILE, "r"))) {
			if (fscanf(pidfp, "%d", &logsrvpid) != 1) ;
			(void) fclose(pidfp);
			if (logsrvpid && !kill(logsrvpid, 0)) {
				(void) filewrt(2, "%s is already running\n", progname);
				return (0);
			} else
			if (logsrvpid && errno == EACCES) {
				(void) filewrt(2, "%s is already running\n", progname);
				return (1);
			}
		}
		switch (logsrvpid = fork())
		{
		case -1:
			filewrt(2, "fork: %s\n", strerror(errno));
			return (1);
		case 0:
			(void) setsid();
			(void) close(0);
			(void) close(1);
			(void) close(2);
			logsrvpid = getpid();
			break;
		default:
			if ((pidfp = fopen(PIDFILE, "w"))) {
				(void) fprintf(pidfp, "%d", logsrvpid);
				(void) fclose(pidfp);
				return(0);
			} else {
				filewrt(2, "%s: %s\n", PIDFILE, strerror(errno));
				_exit(1);
			}
		}
	} else
		logsrvpid = getpid();
	(void) signal(SIGTERM, SigTerm);
	(void) signal(SIGCHLD, (void (*)()) SigChild);
	if (!(ptr = getenv("PORT")))
		ptr = PORT;
	if ((bindfd = tcpbind("0", ptr, 5)) == -1) {
		(void) filewrt(2, "tcpsockbind failed for port %s\n", ptr);
		_exit (1);
	}
	signal(SIGTERM, SigTerm);
	for (;;) {
		if ((sfd = accept(bindfd, (struct sockaddr *) &remotaddr, (socklen_t *) &inaddrlen)) == -1) {
			/* Got signal. */
			switch (errno)
			{
			case EINTR:
				continue;
				/*
				 * have to add more cases here
				 */
			default:
				(void) filewrt(2, "accept: %s\n", strerror(errno));
				if (!foreground)
					(void) unlink(PIDFILE);
				_exit(1);
			}
		}
		switch (pid = fork())
		{
		case -1:
			(void) filewrt(2, "fork: %s\n", strerror(errno));
			(void) close(sfd);
		case 0:
			(void) signal(SIGTERM, SIG_DFL);
			(void) signal(SIGCHLD, SIG_IGN);
			(void) signal(SIGHUP, SIG_IGN);
			(void) close(bindfd);
			if (setsockopt(sfd, SOL_SOCKET, SO_LINGER, (char *) &linger, sizeof(linger)) == -1)
			{
				close(sfd);
				exit(1);
			}
			for (;;)
			{
				if (setsockopt(sfd, SOL_SOCKET, SO_SNDBUF, (void *) &len, sizeof(int)) == -1)
				{
					if (errno == ENOBUFS)
					{
						usleep(1000);
						continue;
					}
					close(sfd);
					exit(1);
				}
				break;
			}
			for (;;)
			{
				if (setsockopt(sfd, SOL_SOCKET, SO_RCVBUF, (void *) &len, sizeof(int)) == -1)
				{
					if (errno == ENOBUFS)
					{
						usleep(1000);
						continue;
					}
					close(sfd);
					exit(1);
				}
				break;
			}
			dup2(sfd, 0);
			dup2(sfd, 3);
			dup2(sfd, 4);
			if (sfd != 0 && sfd != 3 && sfd != 4)
				close(sfd);
#ifdef HAVE_SSL
			if (usessl == 1)
			{
				int             n;

				if (pipe(pi1) != 0 || pipe(pi2) != 0 || pipe(pi3) != 0)
				{
					fprintf(stderr, "unable to create pipe: %s\n", strerror(errno));
					exit(1);
				}
				switch (fork())
				{
				case 0:
					close(pi1[1]);
					close(pi2[0]);
					close(pi3[0]);
					if (dup2(pi1[0], 0) == -1 || dup2(pi2[1], 3) == -1 || dup2(pi3[1], 4) == -1)
					{
						fprintf(stderr, "unable to set up descriptors: %s\n", strerror(errno));
						exit(1);
					}
					if (pi1[0] != 0)
						close(pi1[0]);
					if (pi2[1] != 3)
						close(pi2[1]);
					if (pi3[1] != 4)
						close(pi1[1]);
					/*
				 	* signals are allready set in the parent 
				 	*/
					(void) server(silent);
					close(0);
					close(3);
					close(4);
					_exit(1);
				case -1:
					fprintf(stderr, "%d: unable to fork: %s\n", getpid(), strerror(errno));
					break;
				default:
					break;
				}
				close(pi1[0]);
				close(pi2[1]);
				close(pi3[1]);
				if (!(ssl = SSL_new(ctx)))
				{
					long e;
					while ((e = ERR_get_error()))
						fprintf(stderr, "%d: %s\n", getpid(), ERR_error_string(e, 0));
					fprintf(stderr, "%d: unable to set up SSL session\n", getpid());
					SSL_CTX_free(ctx);
					_exit(1);
				}
				SSL_CTX_free(ctx);
#ifndef CRYPTO_POLICY_NON_COMPLIANCE
				if (!ptr)
					ptr = "PROFILE=SYSTEM";
				if (!SSL_set_cipher_list(ssl, ptr))
				{
					fprintf(stderr, "unable to set ciphers: %s: %s\n", ptr,
						ERR_error_string(ERR_get_error(), 0));
					SSL_free(ssl);
					return (1);
				}
#endif
				if (!(sbio = BIO_new_socket(0, BIO_NOCLOSE)))
				{
					fprintf(stderr, "%d: unable to set up BIO socket\n", getpid());
					SSL_free(ssl);
					_exit(1);
				}
				SSL_set_bio(ssl, sbio, sbio); /*- cannot fail */
				n = translate(ssl, pi1[1], pi2[0], pi3[0], 3600);
				SSL_shutdown(ssl);
				SSL_free(ssl);
				for (retval = -1;(r = waitpid(pid, &status, WNOHANG | WUNTRACED));)
				{
#ifdef ERESTART
					if (r == -1 && (errno == EINTR || errno == ERESTART))
#else
					if (r == -1 && errno == EINTR)
#endif
						continue;
					if (WIFSTOPPED(status) || WIFSIGNALED(status))
					{
						fprintf(stderr, "%d: killed by signal %d\n", pid, WIFSTOPPED(status) ? WSTOPSIG(status) : WTERMSIG(status));
						retval = -1;
					} else
					if (WIFEXITED(status))
					{
						retval = WEXITSTATUS(status);
						fprintf(stderr, "%d: normal exit return status %d\n", pid, retval);
					}
					break;
				} /*- for (; pid = waitpid(-1, &status, WNOHANG | WUNTRACED);) -*/
				if (n)
					_exit(n);
				if (retval)
					_exit(retval);
				_exit (0);
			} else /*- if (usessl == 1) */
			{
				(void) server(silent);
				_exit(1);
			}
#else
		(void) server(silent);
		_exit(1);
#endif
		default:
			(void) close(sfd);
		}
	}	/* for (;;) */
}

int
write_bytes(int fd, umdir_t *Bytes)
{
	if (lseek(fd, sizeof(pid_t), SEEK_SET) == -1) {
		(void) filewrt(2, "lseek: %s\n", strerror(errno));
		return (-1);
	}
	if (write(fd, (char *) Bytes, sizeof(umdir_t)) == -1) {
		(void) filewrt(2, "write_bytes: %s\n", strerror(errno));
		return (-1);
	}
	return (0);
}

#define MAXNOBUFRETRY           60 /*- Defines maximum number of ENOBUF retries -*/
#define SELECTTIMEOUT           30 /*- secs after which select will timeout -*/

int
sockread(fd, buffer, len)
	int             fd;
	char           *buffer;
	int             len;
{
	char           *ptr;
	int             rembytes, rbytes, retrycount;

	for (retrycount = 0, rembytes = len, ptr = buffer; rembytes;)
	{
		errno = 0;
		if ((rbytes = read(fd, ptr, rembytes)) == -1)
		{
#ifdef ERESTART
			if (errno == EINTR || errno == ERESTART)
#else
			if (errno == EINTR)
#endif
				continue;
			if (errno == ENOBUFS && retrycount++ < MAXNOBUFRETRY)
			{
				usleep(1000);
				continue;
			}
#if defined(HPUX_SOURCE)
			if (errno == EREMOTERELEASE)
			{
				rbytes = 0;
				break;
			}
#endif
			return (-1);
		} else
		if (!rbytes)	/* EOF */
			break;;
		rembytes -= rbytes;
		if (!rembytes)
			break;
		ptr += rbytes;
	}
	return (len - rembytes);
}

int
server(int silent)
{
	int             fd, retval, count;
	umdir_t         bytes;
	ssize_t         n;
	pid_t           pid;
	FILE           *socketfp;
	char            buffer[MAXBUF], hostname[56];
	char           *ptr, *(parm[1]);

	if ((retval = sockread(0, hostname, MAXHOSTNAMELEN)) == -1) {
		(void) filewrt(2, "read: %s\n", strerror(errno));
		if (write(3, "1", 1) == -1) ;
		return (1);
	} else
	if (!retval)
		return(1);
	hostname[retval] = 0;
	if (!(socketfp = fdopen(0, "r"))) {
		(void) filewrt(2, "fdopen: %s\n", strerror(errno));
		if (write(3, "1", 1) == -1) ;
		return (1);
	}
	if (write(3, "1", 1) == -1) ;
	(void) sprintf(statusfile, "%s/%s.status", statusdir, hostname);
	if (!access(statusfile, R_OK)) {
		if ((fd = open(statusfile, O_RDWR, 0644)) == -1) {
			(void) filewrt(2, "creat: %s %s\n", statusfile, strerror(errno));
			return(1);
		}
		if ((n = read(fd, (char *) &pid, sizeof(pid_t))) == -1) {
			(void) filewrt(2, "readpid: %s: %s\n", statusfile, strerror(errno));
			return(1);
		}
		if (n == sizeof(pid_t)) {
			if ((n = read(fd, (char *) &bytes, sizeof(umdir_t))) == -1) {
				(void) filewrt(2, "readbytes: %s: %s\n", statusfile, strerror(errno));
				return(1);
			}
		}
	} else {
		if ((fd = open(statusfile, O_CREAT|O_RDWR, 0644)) == -1) {
			(void) filewrt(2, "creat: %s %s\n", statusfile, strerror(errno));
			return(1);
		}
		bytes = 0;
	}
	if (lseek(fd, 0, SEEK_SET) == -1) {
		(void) filewrt(2, "lseek: %s\n", strerror(errno));
		return (-1);
	}
	pid = getpid();
	if (write(fd, (char *) &pid, sizeof(pid_t)) != sizeof(pid_t)) {
		(void) filewrt(2, "write: %s: %s\n", statusfile, strerror(errno));
		return(1);
	}
	filewrt(1, "Connection request from %s\n", hostname);
	(void) signal(SIGTERM, SigTerm);
	for (;; sleep(5)) {
		for (;;) {
			if (!fgets(buffer, MAXBUF - 1, socketfp) || feof(socketfp)) {
				(void) filewrt(2, "client terminated on %s\n", hostname);
				shutdown(0, 0);
				return (1);
			}
			if (rpcflag) {
				*parm = buffer;
				if (log_msg(parm) == -1) {
					shutdown(0, 0);
					return(1);
				}
			} 
			if (!silent && filewrt(1, "%s", buffer) == -1) {
				(void) filewrt(2, "filewrt: out: %s\n", strerror(errno));
				shutdown(0, 0);
				return(1);
			}
			for (count = 0,ptr = buffer;*ptr;ptr++) {
				if (*ptr == ' ')
					count++;
				if (count == 2)
					break;
			}
			if (*ptr)
			{
				bytes += strlen(ptr + 1);
			} else {
				bytes += strlen(buffer);
				filewrt(2, "error-%s", ptr+1);
			}
			if (write_bytes(fd, &bytes) == -1) {
				(void) filewrt(2, "write_bytes: %s\n", strerror(errno));
				shutdown(0, 0);
				return(1);
			}
		} /* for (;;) */
	} /* for (;; sleep(5))*/
}

int
log_msg(char **str)
{
	int            *ret;
	static int      flag;

	if (!flag) {
		if (!(cl = clnt_create(loghost, RPCLOG, CLOGVERS, "tcp"))) {
			filewrt(2, "clnt_create: %s\n", clnt_spcreateerror(loghost));
			return (-1);
		}
		flag++;
	}
	if (!(ret = send_message_1(str, cl))) {
		clnt_perror(cl, "send_message_1");
		clnt_destroy(cl);
		flag = 0;
		return (-1);
	} else
		return (0);
}

int            *
send_message_1(char **argp, CLIENT *clnt)
{
	static int      res;
	struct timeval  TIMEOUT = {25, 0};

	(void) memset((char *) &res, 0, sizeof(res));
	if (clnt_call(clnt, SEND_MESSAGE, (xdrproc_t) xdr_wrapstring, (char *) argp,
			(xdrproc_t) xdr_int, (char *) &res, TIMEOUT) != RPC_SUCCESS) {
		filewrt(2, "clnt_call: %s\n", clnt_sperror(clnt, loghost));
		return (NULL);
	}
	return (&res);
}

void
SigTerm()
{
	if (logsrvpid == getpid()) {
		filewrt(2, "Sending SIGTERM to PROCESS GROUP %d\n", logsrvpid);
		(void) signal(SIGTERM, SIG_IGN);
		kill(-logsrvpid, SIGTERM);
	}
	/*-
	else
	if (!access(statusfile, F_OK))
		(void) unlink(statusfile);
	*/
	exit(0);
}

#ifndef	lint
void
getversion_logsrv_c()
{
	printf("%s\n", sccsid);
}
#endif
